from appJar import gui
import os
import time
import thread
from threading import Thread
from opt_parser import manage_local, manage_conf_update, manage_show_conf
from free_space_checker import check_disk_available



class Gui:
    def __init__(self):
        self.app = gui(handleArgs=False)
        self.create_disk_subwindow()
        self.configure_main_window()
        self.create_progress_subwindow()
        self.enough_proceed_disk_space = None
        #self.p_bar_thread = Thread(target=self.loop_progress)
        #self.p_bar_thread.start()
        self.space_available = 0
        self.app.go()

    def button_manager_main(self, button):
        if button == "Validate file":
            path = self.app.getEntry("file_path")
            manage_conf_update(None, path)
        elif button == "Show configuration":
            manage_show_conf(None, no_check=True)
            self.app.infoBox("Take a look Closer", "Configuration file status displayed in console.")
        if button == "Tar Me":

            self.space_available, enough_space = check_disk_available(gb_availabe_min=1)
            if enough_space is False:
                self.enough_proceed_disk_space = None
                self.app.openSubWindow("disk sub")
                self.app.setLabel("disk_label", "Warning! Only " + str(self.space_available)
                                  + " GB of disk space available. Would you like to proceed ?")
                self.app.showSubWindow("disk sub")
                return

            self.make_tar_ball()

    def make_tar_ball(self):
        description_file_path = self.get_description()
        self.app.showSubWindow("sub1")
        manage_local(None, no_check=True, description_file=description_file_path, keep_raw=False)
        self.app.hideSubWindow("sub1")
        self.app.infoBox("success", "Success Fully created Tarball, Read console log for more details")
        self.app.clearTextArea("pb_describe")
        self.app.setTextArea("pb_describe", "Describe your problem")

    def save_on_low_disk_space(self, button):
        self.app.hideSubWindow("disk sub")
        if button == "Continue":
            self.make_tar_ball()
        else:
            print "Cancel"

    def create_progress_subwindow(self):
        self.app.startSubWindow("sub1", title="Please Wait...", modal=False, transient=True, blocking=False)
        self.app.addDualMeter("progress_meter")
        self.app.setSize(200, 300)
        self.app.stopSubWindow()

    def create_disk_subwindow(self):
        self.app.startSubWindow("disk sub", modal=True)
        self.app.addLabel("disk_label", "")
        self.app.addButtons(["Continue", "Cancel"], self.save_on_low_disk_space)
        self.app.stopSubWindow()

    def proceed_disk_space(self):
        print("toto")
        self.enough_proceed_disk_space = True
        self.app.hideSubWindow("disk_sub")

    def cancel_disk_space(self):
        print("titi")
        self.enough_proceed_disk_space = False
        self.app.hideSubWindow("disk_sub")

    def loop_progress(self):
        while self.app:
            for i in range(0, 100):
                self.app.setMeter("progress_meter", [i, i])
                time.sleep(0.3)

    def configure_main_window(self):
        self.app.addTextArea("pb_describe")
        self.app.setTextArea("pb_describe", "Describe your problem")
        self.app.addButtons(["Show configuration", "Tar Me"], self.button_manager_main)
        self.app.addFileEntry("file_path")
        self.app.addButton("Validate file", self.button_manager_main)

    def get_description(self):
        input = self.app.getTextArea("pb_describe")
        if(input != "Describe your problem"):#here we should save in file
            description_file_name = "bug describer"
            fp = open(description_file_name, "w")
            fp.write(input)
            fp.close()
            cwd = os.getcwd()
            file_path = cwd + "/" + description_file_name
            return file_path
        return None



import colors
from gopher_robot_version import sys_packages
from gopher_robot_version import ros_packages
from gopher_robot_version import writer
import os

def prepare_string():
    result = ""
    error_msg = " packages directory NOT FOUND, and won't be part of gopher_version_file"
    success_msg =" packages version"+ colors.bcolors.OKGREEN + " successfully" + colors.bcolors.ENDC + \
                 " added to gopher_version_file"

    devel_path = "/opt/yujin/amd64/indigo-devel"
    stable_path = "/opt/yujin/amd64/indigo-stable"
    base_path = "/opt/ros"

    system_dict = sys_packages.get_system_packages()
    print( "System" + success_msg)


    try:
        ros_dict_devel = ros_packages.get_ros_packages(devel_path)
        print (devel_path + success_msg)
    except OSError:
        print(colors.bcolors.FAIL + "-------->  " + colors.bcolors.ENDC
              + devel_path + error_msg)
        ros_dict_devel = {}

    try:
        ros_dict_stable = ros_packages.get_ros_packages(stable_path)
        print(stable_path + success_msg)
    except OSError:
        print(colors.bcolors.FAIL + "-------->  " + colors.bcolors.ENDC
              + stable_path + error_msg)
        ros_dict_stable = {}

    try:
        ros_dict_base = ros_packages.get_ros_packages(base_path)
        print(base_path + success_msg)
    except OSError:
        print(colors.bcolors.FAIL + "-------->  " + colors.bcolors.ENDC
              + base_path + error_msg)
        ros_dict_base = {}


    result += writer.dict_to_str(system_dict, "SYSTEM PACAKGES")
    result += writer.dict_to_str(ros_dict_devel, "YUJIN_DEVEL")
    result += writer.dict_to_str(ros_dict_stable, "YUJIN STABLE")
    result += writer.dict_to_str(ros_dict_base, "ROS INTERNAL")

    return result

def write_to_file():
    filename = "gopher_version_file"
    fp = open(filename, "w+")
    data = prepare_string()
    fp.write(data)
    fp.close()
    cwd = os.getcwd()
    file_path = cwd + "/" + filename
    return file_path



import psutil


def check_disk_available(gb_availabe_min=1):
    available_space = psutil.disk_usage('/')[2]
    available_space_gb = available_space * 10e-10
    return available_space_gb, available_space_gb > gb_availabe_min


import argparse
import conf_loader
import tar_manager

def pars_args():
    parser = argparse.ArgumentParser(description="A simple, configurable software to tar a set of preconfigured log "
                                                 "files.", prog='log_aggregator', version='0.50')
    parser.add_argument("--show_conf", action="store_true", default=False, help="Show the actual configuration file in"
                                                                                " use if it exists")

    parser.add_argument("--default_conf", action="store_true", default=False, help="WARNING: Delete your actual conf if"
                                                                                   " it exists and generate the default"
                                                                                   " configuration file")
    parser.add_argument("--local", action="store_true", default=False, help="Create a local tarball under home directory")
    parser.add_argument("--gui", action="store_true", default=False, help="A gui to help you")
    parser.add_argument("--add_file", action="store_true", default=False, help="Add a file path in configuration file")
    parser.add_argument("--keep_raw", action="store_true", default=False, help="Allows to keep raw file stacks "
                                                                               "containing videos taken by 3d sensors.")
    return parser.parse_args()

def main():
    parsed_args = pars_args()
    manage_gui(parsed_args)
    manage_local(parsed_args, keep_raw=parsed_args.keep_raw)
    manage_show_conf(parsed_args)
    manage_default_conf(parsed_args)

def manage_gui(parsed_args):
    if(parsed_args.gui):
        import gui_test
        gui = gui_test.Gui()

def manage_conf_update(parsed_args, file_to_add=None):
    if(file_to_add is not None):
        conf_loader.add_entry_to_conf_file(file_to_add)
    elif(parsed_args.add_file):
       conf_loader.add_entry_to_conf_file(parsed_args.add_file)


def manage_local(parsed_args, no_check=False, description_file=None, keep_raw=False):
    run = no_check


    if(run == False and parsed_args.local):
        run = True

    if (run):
        file_list = conf_loader.check_create_load_conf(keep_raw=keep_raw)
        if description_file:
            file_list.append(description_file)
        tar_path, folder_to_tar_path = tar_manager.prepare_tar(file_list)
        tar_manager.clean_up(None, folder_to_tar_path)


def manage_show_conf(parsed_args, no_check=False):
    if (no_check or parsed_args.show_conf):
        if (conf_loader.check_conf_file_exist()):
            conf_loader.check_create_load_conf(skip_check=True, show_mode=True)


def manage_default_conf(parsed_args):
    if (parsed_args.default_conf):
        conf_loader.generate_default_conf()



import colors
import distutils
from distutils.dir_util import copy_tree
import os
from os.path import expanduser
import pycurl



from shutil import copyfile, rmtree
import socket
import tarfile
import time

def prepare_file_list(found_file_list):
    """
    Some path are relative, we make them absolute
    :param found_file_list: list of paths
    :return: The input list with only absolute paths
    """
    real_path = []
    for element in found_file_list:
        real_path.append(expanduser(element))
    print real_path

    return real_path


def generate_filename():
    """
    Generate the name of the tar file
    :return: the name string
    """
    name = time.strftime("%y.%m.%d_%H.%M.%S")
    name += "_" + socket.gethostname()
    return name

def gen_cpu_usage():
    os.system("top -n4 -b -c > " + "~/cpu_usage")

def prepare_tar(file_list):
    """

    :param file_list:
    :return:
    """
    gen_cpu_usage()
    file_list.append(expanduser("~/cpu_usage"))
    name = generate_filename()
    folder_to_tar_path = expanduser("~/") + name + "/" #folder to be tared
    os.makedirs(folder_to_tar_path)

    for file_path in file_list:
        try:
            real_path = expanduser(file_path)
            folder_name = real_path.split("/")[-1]
            copy_tree(real_path, folder_to_tar_path + folder_name)
        except distutils.errors.DistutilsFileError:
            filename = (real_path.split("/"))[-1]
            copyfile(real_path, folder_to_tar_path + "/" + filename)

    tar_name = tar_folder(folder_to_tar_path, name)
    print (colors.bcolors.OKGREEN + "Tar-ball successfully created under: "+ colors.bcolors.ENDC + tar_name)
    return tar_name, folder_to_tar_path

def clean_up(tar, folder):
    """
    Delete the tar ball and the folder used to make it.

    :param tar: Path to the tarball file, set to none if we do not want to remove it
    :param folder:
    :return:
    """
    print(colors.bcolors.OKBLUE + "Starting clean up..." + colors.bcolors.ENDC)
    print(colors.bcolors.OKBLUE + "The following files will be deleted from local system:" + colors.bcolors.ENDC)
    if (tar):
        print(colors.bcolors.FAIL + "-------->  " + colors.bcolors.ENDC + tar)
    if (folder):
        print(colors.bcolors.FAIL + "-------->  " + colors.bcolors.ENDC + folder)

    if (tar):
        os.remove(tar)
    if (folder):
        rmtree(folder)
    print(colors.bcolors.OKGREEN + "Files successfully deleted from local system..." + colors.bcolors.ENDC )


def tar_folder(folder_path, name):
    """
    Take an folder and make a tarball of it
    :param folder_path: Path to the folder to tar
    :param name: name of the output tar file desired
    :return: The tarball path
    """
    tar_name = name + ".tar.bz2"
    tar_name = expanduser("~/" + tar_name)
    with tarfile.open(tar_name, "w:bz2") as tar:
        tar.add(folder_path, arcname=os.path.basename(folder_path))
    return tar_name


from PIL import Image
from tqdm import tqdm
#dirty workaround to fix api bug
tqdm.monitor_interval = 0

import os
import re
import struct




class Img_stack_wrapper():
        def __init__(self, size_x, size_y, data_type, camera_type, stack_path,
                     frame_offset=0, endianess=None):
            """

            :param size_x: Width of one frame of the frame stack/raw file
            :param size_y: Height of one frame of the frame stack/ raw file
            :param data_type: Give information on the type of data composing one pixel of the image, for now only
            support rgb8
            :param camera_type: Let us know which camera of the robot outputed the raw file assigned to this wrapper
            object
            :param stack_path: Path to the  raw file assigned to this wrapper
            :param frame_offset: If some data is interlaced between frames, we inform here the size of this data
            :param endianess: optional and not used for now but can be relevant for further developments
            """
            self.valid = True
            self.camera_type = camera_type

            self.size_x = size_x
            self.size_y = size_y
            self.stack_path = stack_path

            if data_type == 'rgb8':
                self.bytes_per_pixel = 3
                self.data_type = 'rgb'
            else:
                raise Exception("Type error: %s is an unknown data type" % (data_type))

            try:
                self.fp = open(stack_path, "rb")
            except IOError:
                print("Video stack file not found")
                self.valid = False

            if self.valid:
                self.stack_size_byte = os.path.getsize(self.stack_path)
                self.pixel_per_frame = self.size_x * self.size_y
                self.byte_per_frame = self.pixel_per_frame * self.bytes_per_pixel
                self.frame_offset = frame_offset
                self.valid_frames = self.stack_size_byte / (self.byte_per_frame + self.frame_offset)
                self.endianess = endianess
            else:
                self.valid_frames = 0
            self.read = 0

        def __del__(self):
            """
            If the assigned raw file to a wrapper instance is considered as correct a file pointer is opened on this
            same raw file, we need to close it at object destruction
            """
            if self.valid:
                self.fp.close()

        def get_nb_read_frames(self):
            """
            Getter
            :return: the number of frames already requested read by the wrapper from the raw file
            """
            return str(self.read)

        def get_next_raw_img_and_extra_data(self):
            """
            return the next raw image data and if the interlacing data exist in the raw file assigned to this wrapper
            it will be also returned
            :return: next raw frame data or next raw frame data and next raw i
            """
            if (self.read < self.valid_frames):
                img_binary = self.fp.read(self.byte_per_frame)
                self.read += 1
                if (self.frame_offset > 0):
                    extra_data = self.fp.read(self.frame_offset)
                    return img_binary, extra_data
                return img_binary
            if (self.frame_offset > 0):
                return None, None
            else:
                return None

        def get_percent_read(self):
            """
            Getter
            :return: The percent of frames already read with the method get_next_raw_img_and_extra_data
            """
            return (self.read / float(self.valid_frames)) * 100

        def is_valid(self):
            """
            Getter
            :return: Let the object user know if the raw data assigned to the wrapper is valid to be used, this of
            course does not give any warranty on the data quality
            """
            return self.valid

        def get_camera_type(self):
            """
            Getter
            :return: The camera name of the robot which outputed the raw file assigned to this wrapper instance
            """
            return self.camera_type


def gen_stack_wrapper(video_stack_path):
    """
    Assing and configure a Img_stack_wrapper object with the given raw file/stack path given as argument
    :param video_stack_path: path to the raw file/stack containing frames from video. The raw filename should be formated as
    video_CAMERATYPE_FRAMEWIDTH_FRAMEHEIGHT_ENDIANESS_TYPEOFDATA
    The only type of data supported for now is rgb8
    :return: A configured Img_stack_wrapper object configured with the given raw file/stack path given as argument
    """
    # number of info contained in filename
    info_count = 6
    # size of data interlaced beetween frames, for now according to our usage a hard setted value is enough, it can
    # become a function argument for further developments
    offset = 8
    stack_filename = video_stack_path.split('/')[-1]
    info_list = stack_filename.split('_')

    if len(info_list) != info_count:
        return None

    camera_type = info_list[1]
    size_x = int(info_list[2])
    size_y = int(info_list[3])
    endianess = info_list[4]
    data_type = info_list[5]
    stack_wrapper = Img_stack_wrapper(size_x, size_y, data_type, camera_type, video_stack_path,
                                frame_offset=offset, endianess=endianess)
    return stack_wrapper


def get_video_file_path_list(log_path):
    """
    Allow to gather a list of path of file contained in the log_path following the "video_*" patern
    :param log_path: Path where to search the video files, in our case it will be always ros log path
    :return: a list of paths corresponding to raw "video" files
    """
    log_content = os.listdir(log_path)
    regex = re.compile('video_*')
    video_stack_path_list = []
    for filename in log_content:
        if (regex.search(filename)):
            video_stack_path_list.append(log_path + '/' + filename)

    return video_stack_path_list


def gen_stack_wrapper_list(video_stack_path_list):
    """
    Generate a Img_stack_wrapper obect for each path contained in the video_stack_path_list.
    :param video_stack_path_list: A list of paths to video files
    :return: A list of Img_stack_wrapper objects
    """
    wrapper_list = []
    for element in video_stack_path_list:
        wrapper = gen_stack_wrapper(element)
        wrapper_list.append(wrapper)
    return wrapper_list

def gen_images_from_wrapper_list(img_stack_wrapper_list, log_path, extra_data_present=False):
    """
    Launch the image generation process for each valid Img_stack_wrapper object contained in the img_stack_wrapper_list
    argument.
    :param img_stack_wrapper_list: A list of Img_stack_wrapper objects already configured to a raw/stack video file
    :param log_path: Path where the wrapper generated images will be outputed, named log_path because in our case
    it should alays be in the ros log directory, naming can be change upon further developments
    :param extra_data_present: Indicate if extra data as been inserted between frames
    """
    for wrapper in img_stack_wrapper_list:
        if wrapper.is_valid():
            print ('Unwarping ' + wrapper.get_camera_type() + '\n')
            gen_images_from_wrapper(wrapper, log_path, extra_data_present=extra_data_present)


def gen_images_from_wrapper(img_stack_wrapper, log_path, extra_data_present=False):
    """
    Create a directory named as Img_stack_wrapper camera_type attribute
    Request all available frames from the Img_stack_wrapper object and save them as png files under the previously
    created directory
    The file naming follow this pattern
    FRAMENUMBER(-EXTRADATA).png
    :param img_stack_wrapper: A valid img_stack_wrapper object
    :param log_path: Path where the wrapper generated images will be outputed, named log_path because in our case
    it should always be in the ros log directory, naming can be change upon further developments
    :param extra_data_present: Indicate if extra data as been inserted between frames
    """
    img_path = log_path + '/' + img_stack_wrapper.camera_type
    subdir_exist = os.path.exists(img_path)

    if (extra_data_present):
        raw_img, extra_data = img_stack_wrapper.get_next_raw_img_and_extra_data()
    else:
        raw_img = img_stack_wrapper.get_next_raw_img_and_extra_data()

    pbar = tqdm(total=100)
    last_percent_read = 0

    while raw_img is not None:
        if not subdir_exist:
            os.mkdir(img_path)
            subdir_exist = True

        filename = img_stack_wrapper.get_nb_read_frames() + "_"
        if (extra_data_present):
            filename += str(struct.unpack('d', extra_data)[0])
        filename += ".png"

        dest_path = img_path + '/' + filename
        pil_img = Image.frombytes("RGB", (img_stack_wrapper.size_x, img_stack_wrapper.size_y), raw_img, decoder_name='raw')
        pil_img.save(dest_path, optimize=True)
        
        new_percent_read = img_stack_wrapper.get_percent_read()
        progress = new_percent_read - last_percent_read
        last_percent_read = new_percent_read
        pbar.update(progress)

        if (extra_data_present):
            raw_img, extra_data = img_stack_wrapper.get_next_raw_img_and_extra_data()
        else:
            raw_img = img_stack_wrapper.get_next_raw_img_and_extra_data()

    del(pbar)
    print('\n')


def convert_video_to_image(log_path, keep_raw):
    """
    Find raw/stack video files, and convert their content to png files under subdirectories named depending the raw file
    name.
    :param log_path: Path where to search for raw/stack video files, named log_path because in our case
    it will always be the path to the ros log directory, naming can be change upon further developments
    """
    video_stack_path_list = get_video_file_path_list(log_path)
    wrapper_list = gen_stack_wrapper_list(video_stack_path_list)
    gen_images_from_wrapper_list(wrapper_list, log_path, extra_data_present=True)
    if keep_raw is False:
        for file in video_stack_path_list:
            os.remove(file)




from setuptools import setup

setup(name='log_aggregator',
      version='0.50',
      description='Create a configurable Tarball of various files in the file system',
      author='Thomas Kostas',
      url='https://github.com/yotabits/log_aggregator',
      author_email='tkostas75@gmail.com',
      license='MIT',
      packages=['log_aggregator'],
      scripts=['bin/log_aggregator'],
      install_requires=['pycurl',
                        'appjar',
                        'gopher_robot_version',
                        'pillow',
                        'tqdm',
                        'psutil'
      ],
      classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
        ],
      zip_safe=False)



import colors
from os.path import expanduser
import os
from specific_entries_manager import manage_entries

def check_file_existence(files_list):
    """
    The existence or not of files or DIRECTORIES paths in file list
    when an invalid path is found it's automatically remove from the list.
    :param files_list: List of paths to check for existence.
    :return: Return the input list with incorrect paths removed as first return value. As a second value return
    the list of incorrect paths
    """
    not_exist_list = []
    for file in files_list:
        tmp_file = expanduser(file)
        if not os.path.exists(tmp_file):
            if not os.path.isfile(tmp_file):
                not_exist_list.append(file)

    for file in not_exist_list:
        files_list.remove(file)

    return files_list, not_exist_list


def check_create_load_conf(skip_check=False, show_mode=False, keep_raw=False):
    """
    Firstly check if the configuration file ~/.log_to_nurc exist, create it if not, read the files to tar
    from this configuration file and check their existence.
    :param skip_check: configuration file existence checking is skipped if this is set to true
    :return: Return the file list read from log_to_nucrc files/path, ensure that each of these path exists
    """
    conf_file_found = True
    conf_file_path = get_default_conf_path()

    if (skip_check == False):
        conf_file_found = check_conf_file_exist(conf_file_path)

    files_list = []
    if(conf_file_found):
        conf_file_fp = open(conf_file_path)
        line = conf_file_fp.readline()
        while line:
            line = line.replace('\n', '')
            entry_files = manage_entries(line, show_mode=show_mode, keep_raw=keep_raw)
            for file in entry_files:
                files_list.append(file)
            line = conf_file_fp.readline()
    else:
        generate_default_conf(conf_file_path)
        print(colors.bcolors.OKGREEN + "Configuration file successfully created..." + colors.bcolors.ENDC)
        return check_create_load_conf()

    files_list, not_exist_list = check_file_existence(files_list)

    if(show_mode and "gopher_version_file" in not_exist_list):
        not_exist_list.remove("gopher_version_file")
        files_list.append("gopher_version_file")

    file_sent_displayer(files_list, not_exist_list)
    return files_list


def file_sent_displayer(files_list, not_exist_list):
    """
    Simply pretty print the files/folder that will be successfully sent and the one that does not exists.

    :param files_list: List of files/folders to be tared and sent.
    :param not_exist_list: List of files/folders found in configuration file that does not exists.
    """
    print(colors.bcolors.OKGREEN + "The following files will be tared:" + colors.bcolors.ENDC)
    for file in files_list:
        print (colors.bcolors.OKGREEN + "-------->  " + colors.bcolors.ENDC +  file)
    if len(not_exist_list) != 0:
        print colors.bcolors.FAIL + "The following file(s) won't be tared as they DO NOT exist under the curent file system:" + colors.bcolors.ENDC
        for file in not_exist_list:
            print (colors.bcolors.FAIL + "-------->  "+ colors.bcolors.ENDC + file)
        print colors.bcolors.FAIL + "Hint: To prevent this message from appearing again correct the previously listed " \
                                    "entries under ~/.log_to_nucrc configuration file." + colors.bcolors.ENDC


def check_conf_file_exist(conf_file_path=None):
    """
    Check if the ~/.log_to_nucrc file exist
    :param conf_file_path: custom configuration path
    :return: True or False
    """
    if (conf_file_path is None):
        conf_file_path = get_default_conf_path()

    if (os.path.isfile(conf_file_path)):
        print(colors.bcolors.OKGREEN + "Configuration file found..." + colors.bcolors.ENDC)
        return True
    else:
        print(colors.bcolors.WARNING + "Configuration file not found..." + colors.bcolors.ENDC)
        return False


def generate_default_conf(conf_file_path=None):
    """
    Create the default configuration file.
    :param conf_file_path: custom path for configuration file
    :return: return the list of files to avoid to read it again.
    """
    if (conf_file_path is None):
        conf_file_path = get_default_conf_path()

    default_file_list = ["latest_log", "~/.gopher_robot_environment.bash", "/var/log/syslog",
                         "gopher_version", "~/.ros/log/console_output.log", "/proc/version"]
    conf_file_fp = open(conf_file_path, "w")
    for element in default_file_list:
        conf_file_fp.write(element + "\n")
    conf_file_fp.close()
    print (colors.bcolors.OKGREEN + "Default configuration file successfully generated ~/.log_to_nucrc"
           + colors.bcolors.ENDC)
    return


def get_default_conf_path():
    """
    Return the absolute path of the configuration file
    :return: configuration file path
    """
    conf_file_name = '.log_to_nucrc'
    conf_file_path = expanduser("~/" + conf_file_name)
    return conf_file_path


def add_entry_to_conf_file(entry_to_add, conf_file_path=None):
    """
    Add an entry to the configuration file
    :param entry_to_add:  absolute or relative path of the file or directory to be added to the configuration file
    :param conf_file_path:
    :return:
    """
    wrote = False

    if (entry_to_add == "" or entry_to_add == None):
        print(colors.bcolors.WARNING + "File path not Provided !" + colors.bcolors.ENDC)
        return wrote

    if (not check_conf_file_exist()):
        generate_default_conf()
    if conf_file_path is None:
        conf_file_path = get_default_conf_path()

    conf_file_fp = open(conf_file_path, "r+")
    read = conf_file_fp.readline()

    found_list = []
    while read:
        found_list.append(read)
        read = conf_file_fp.readline()

    if entry_to_add not in found_list:
        conf_file_fp.writelines(entry_to_add)
        wrote = True
        print(colors.bcolors.OKGREEN + "File path Succesfully append to configuration file..." + colors.bcolors.ENDC)
    else:
        print(colors.bcolors.FAIL + "File path already exist in actuall configuration file.")
    conf_file_fp.close()
    return wrote